#
# Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
"""
This files contains unit tests for the MySQL General and Slow Query Log
parsers.
"""

import sys
import os.path
_HERE = os.path.dirname(os.path.abspath(__file__))
_ROOTPATH = os.path.split(_HERE)[0]
sys.path.append(_ROOTPATH)

import tempfile
import inspect
import datetime
import decimal
import itertools
import unittest

from mysql.utilities.exception import (
    LogParserError,
    )

from mysql.utilities.common.parser import *

LOG_HEADERS = [
    ('/usr/sbin/mysqld, Version: 5.1.54-1ubuntu4-log'
        ' ((Ubuntu)). started with:',
     'Tcp port: 3309  Unix socket: /var/run/mysqld/mysqld3.sock',
     'Time                 Id Command    Argument',),
    ('/usr/local/mysql/bin/mysqld, Version: 5.5.17-log'
        ' (Source Distribution). started with:',
     'Tcp port: 3306  Unix socket: /tmp/mysql.sock',
     'Time                 Id Command    Argument',),
     ('/usr/local/mysql/bin/mysqld, Version: 5.5.17'
         ' (Source Distribution). started with:',
      'Tcp port: 3306  Unix socket: /tmp/mysql.sock',
      'Time                 Id Command    Argument',),
]
LOG_HEADERS_EXP = [
    {'_version': (5,1,54,"1ubuntu4-log"),
     '_program': '/usr/sbin/mysqld',
     '_port': 3309,
     '_socket': '/var/run/mysqld/mysqld3.sock',
     '_start_datetime': None,
     '_last_seen_datetime': None,},
    {'_version': (5,5,17,"log"),
     '_program': '/usr/local/mysql/bin/mysqld',
     '_port': 3306,
     '_socket': '/tmp/mysql.sock',
     '_start_datetime': None,
     '_last_seen_datetime': None,},
    {'_version': (5,5,17,None),
     '_program': '/usr/local/mysql/bin/mysqld',
     '_port': 3306,
     '_socket': '/tmp/mysql.sock',
     '_start_datetime': None,
     '_last_seen_datetime': None,},
]

GENERAL_LOG_ENTRIES = {
    'Connect': (
        "111102  9:48:46\t    3 Connect\troot@localhost on",
        "111102 16:48:46\t    4 Connect\troot@localhost on mysql",
        "\t\t    5 Connect\troot@localhost on mysql",
        ),
    'Query': (
        "111102 12:49:02\t    3 Query\tinsert into t1 values (3),(4)",
        "\t\t    3 Query\tinsert into t1 values (5),(6)",
        "111102 14:49:02\t    4 Query\tinsert into t1\nvalues (7),(8)",
        ),
    'Init DB': (
        "111102 12:49:02\t    3 Init DB\ttest",
        "\t\t    3 Init DB\ttest",
        "\t\t    4 Init DB\ttest",
        ),
    'Quit': (
        "111102 12:49:04\t    3 Quit\t",
        "\t\t    3 Quit\t",
        "\t\t    4 Quit\t",
        ),
    'Prepare': (
        "111206 18:12:23\t    35 Query\tPREPARE stmt1 FROM 'SELECT * FROM t1",
        "WHERE id = ?'",
        "\t\t    35 Prepare\tSELECT * FROM t1",
        "WHERE id = ?",
        ),
}

GENERAL_LOG_ENTRIES_EXP = {
    'Connect': (
        {'database': None,
         'argument': 'root@localhost on',
         'session_id': 3,
         'datetime': datetime.datetime(2011, 11, 2, 9, 48, 46),
         'host': 'localhost',
         'command': 'Connect',
         'user': 'root'},
        {'database': 'mysql',
         'argument': 'root@localhost on mysql',
         'session_id': 4, 
         'datetime': datetime.datetime(2011, 11, 2, 16, 48, 46),
         'host': 'localhost',
         'command': 'Connect',
         'user': 'root'},
        {'database': 'mysql',
         'argument': 'root@localhost on mysql',
         'session_id': 5,
         'datetime': None,
         'host': 'localhost',
         'command': 'Connect',
         'user': 'root'},
        ),
    'Query': (
        {'database': None,
         'argument': 'insert into t1 values (3),(4)',
         'session_id': 3,
         'datetime': datetime.datetime(2011, 11, 2, 12, 49, 2),
         'host': None,
         'command': 'Query',
         'user': None},
        {'host': None,
         'command': 'Query',
         'user': None,
         'database': None,
         'argument': 'insert into t1 values (5),(6)',
         'session_id': 3,
         'datetime': datetime.datetime(2011, 11, 2, 12, 49, 2)},
        {'host': None,
         'command': 'Query',
         'user': None,
         'database': None,
         'argument': 'insert into t1\nvalues (7),(8)',
         'session_id': 4,
         'datetime': datetime.datetime(2011, 11, 2, 14, 49, 2)},
    ),
    'Init DB': (
        {'host': None,
         'command': 'Init DB',
         'user': None,
         'database': 'test',
         'argument': 'test',
         'session_id': 3,
         'datetime': datetime.datetime(2011, 11, 2, 12, 49, 2)},
        {'host': None,
         'command': 'Init DB',
         'user': None,
         'database': 'test',
         'argument': 'test',
         'session_id': 3,
         'datetime': datetime.datetime(2011, 11, 2, 12, 49, 2)},
        {'host': None,
         'command': 'Init DB',
         'user': None,
         'database': 'test',
         'argument': 'test',
         'session_id': 4,
         'datetime': None},
    ),
    'Quit': (
        {'host': None,
         'command': 'Quit',
         'user': None,
         'database': None,
         'argument': None,
         'session_id': 3,
         'datetime': datetime.datetime(2011, 11, 2, 12, 49, 4)},
        {'host': None,
         'command': 'Quit',
         'user': None,
         'database': None,
         'argument': None,
         'session_id': 3,
         'datetime': None},
        {'host': None,
         'command': 'Quit',
         'user': None,
         'database': None,
         'argument': None,
         'session_id': 4,
         'datetime': None},
    ),
    'Prepare': (
        {'host': None,
         'command': 'Query',
         'user': None,
         'database': None,
         'argument': "PREPARE stmt1 FROM 'SELECT * FROM t1\nWHERE id = ?'",
         'session_id': 35,
         'datetime': datetime.datetime(2011, 12, 6, 18, 12, 23)},
        {'host': None,
         'command': 'Prepare',
         'user': None,
         'database': None,
         'argument': 'SELECT * FROM t1\nWHERE id = ?',
         'session_id': 35,
         'datetime': datetime.datetime(2011, 12, 6, 18, 12, 23)},
    ),
}

SLOW_LOG_ENTRIES = [
    ('# Time: 111102 12:48:46',
     '# User@Host: root[root] @ localhost []',
     '# Query_time: 0.000333  Lock_time: 0.000000'
        ' Rows_sent: 1  Rows_examined: 0',
     'SET timestamp=1320234526;',
     'select @@version_comment limit 1;'),
    ('# Time: 111125 11:56:45',
     '# User@Host: root[root] @ localhost []',
     '# Query_time: 12.000333  Lock_time: 34.000000'
        ' Rows_sent: 44345  Rows_examined: 3123423',
     'SET timestamp=1322218605;',
     'select c1',
     'from t1',
     'where c1 like "%a%";'),
    ('# User@Host: root[root] @ localhost []',
     '# Query_time: 2.000333  Lock_time: 2.000000'
        ' Rows_sent: 44342  Rows_examined: 3123428',
     'SET timestamp=1322218605;',
     'select c1',
     'from t1',
     'where c1 like "%a%";'),
    ('# User@Host: a[b] @ c.d [127.0.0.1]',
     '# Query_time: 0.000059  Lock_time: 0.000019'
        'Rows_sent: 3  Rows_examined: 3',
     'SET timestamp=1338440410;',
     'SELECT 1;'),
    ('# User@Host: a[b] @ c.d [127.0.0.1]  Id:     1',
     '# Query_time: 0.000059  Lock_time: 0.000019'
        'Rows_sent: 3  Rows_examined: 3',
     'SET timestamp=1338440410;',
     'SELECT 1;')
]
SLOW_LOG_ENTRIES_EXP = [
    {'query_time': decimal.Decimal('0.000333'),
     'rows_examined': 0,
     'rows_sent': 1,
     'database': None,
     'lock_time': decimal.Decimal('0.000000'),
     'datetime': datetime.datetime.fromtimestamp(1320234526),
     'host': 'localhost',
     'user': 'root',
     'session_id': None,
     'query': 'SET timestamp=1320234526;\nselect @@version_comment limit 1;'},
    {'query_time': decimal.Decimal('12.000333'),
     'rows_examined': 3123423,
     'rows_sent': 44345,
     'database': None,
     'lock_time': decimal.Decimal('34.000000'),
     'datetime': datetime.datetime.fromtimestamp(1322218605),
     'host': 'localhost',
     'user': 'root',
     'session_id': None,
     'query': ('SET timestamp=1322218605;\n'
               'select c1\nfrom t1\nwhere c1 like "%a%";')},
    {'query_time': decimal.Decimal('2.000333'),
     'rows_examined': 3123428,
     'rows_sent': 44342,
     'database': None,
     'lock_time': decimal.Decimal('2.000000'),
     'datetime': datetime.datetime.fromtimestamp(1322218605),
     'host': 'localhost',
     'user': 'root',
     'session_id': None,
     'query': ('SET timestamp=1322218605;\n'
               'select c1\nfrom t1\nwhere c1 like "%a%";')},
    {'query_time': decimal.Decimal('0.000059'),
     'rows_examined': 3,
     'rows_sent': 3,
     'database': None,
     'lock_time': decimal.Decimal('0.000019'),
     'datetime': datetime.datetime.fromtimestamp(1338440410),
     'host': 'c.d',
     'user': 'a',
     'session_id': None,
     'query': 'SET timestamp=1338440410;\nSELECT 1;'},
    {'query_time': decimal.Decimal('0.000059'),
     'rows_examined': 3,
     'rows_sent': 3,
     'database': None,
     'lock_time': decimal.Decimal('0.000019'),
     'datetime': datetime.datetime.fromtimestamp(1338440410),
     'host': 'c.d',
     'user': 'a',
     'session_id': '1',
     'query': 'SET timestamp=1338440410;\nSELECT 1;'}
]

class BaseParserTestCase(unittest.TestCase):
    """Base class for all unittests testing parsers
    """
    def checkArguments(self, function, supported_arguments):
        argspec = inspect.getargspec(function)
        function_arguments = dict(zip(argspec[0][1:],argspec[3]))
        for argument, default in function_arguments.items():
            try:
                self.assertEqual(supported_arguments[argument],
                    default, msg ="Argument '%s' has wrong default" % argument)
            except KeyError:
                self.fail("Found unsupported or new argument '%s'" % argument)
        for argument, default in supported_arguments.items():
            if not function_arguments.has_key(argument):
                self.fail("Supported argument '%s' fails" % argument)
    
    def _fakelog_writelines(self, lines, empty=True, rewind=True):
        """Helper function writing to file object self.fakelog

        The attribute fakelog should be a temporary file. If it is available,
        this function will write all items in the lines-list to the file.
        When empty is true (default), the file will be truncated.

        After writing, the current possition will be rest to the first byte
        unless rewind is set to False.
        """
        try:
            if empty is True:
                self.fakelog.seek(0)
                self.fakelog.truncate()
            self.fakelog.write('\n'.join(lines)+'\n')
            if rewind is True:
                self.fakelog.seek(0)
        except AttributeError:
            pass
    
    def check_attributes(self, obj, expected):
        if isinstance(obj, dict):
            objdict = obj
        else:
            objdict = obj.__dict__
        for key,value in expected.items():
            self.assertTrue(key in objdict,
                msg="No attribute '%s' for instance of class %s" % (
                    key,obj.__class__.__name__))
            self.assertEqual(value, objdict[key])

class TestLogParserBase(BaseParserTestCase):
    """Test LogParserBase-class
    """
    def setUp(self):
        self.fakelog = tempfile.SpooledTemporaryFile(max_size=3*1024)
        self.log = LogParserBase(stream=self.fakelog)
    
    def tearDown(self):
        try:
            self.fakelog.close()
        except:
            pass # Does not matter if self.fakelog can't be closed
    
    def test_init(self):
        """Initialzing LogParserBase instance"""
        self.assertRaises(TypeError,LogParserBase)

        exp = {
            '_stream': self.fakelog,
            '_version': None,
            '_program': None,
            '_port': None,
            '_socket': None,
            '_start_datetime': None,
            '_last_seen_datetime': None,
        }
        self.check_attributes(self.log,exp)
        
        # Check logs not starting with headers
        self._fakelog_writelines(GENERAL_LOG_ENTRIES['Connect'])
        self.log = LogParserBase(self.fakelog)
        self.check_attributes(self.log,exp)
        self.assertEqual(GENERAL_LOG_ENTRIES['Connect'][0],
                         self.log._get_next_line())

        self._fakelog_writelines(LOG_HEADERS[0], empty=True)
        exp = {
            '_stream': self.fakelog,
            '_version': (5,1,54,"1ubuntu4-log"),
            '_program': '/usr/sbin/mysqld',
            '_port': 3309,
            '_socket': '/var/run/mysqld/mysqld3.sock',
            '_start_datetime': None,
            '_last_seen_datetime': None,
        }
        self.log = LogParserBase(self.fakelog)
        self.check_attributes(self.log,exp)

    def test_get_next_line(self):
        """Get the next line from the log file"""
        data = (
            '# Line 1',
            '# Line 2',
            )
        self._fakelog_writelines(data, empty=True)
        self.assertEqual(data[0],self.log._get_next_line())
        self.assertEqual(data[1],self.log._get_next_line())

        # Go to the end of the stream
        self.log._stream.seek(0, os.SEEK_END)
        self.assertEqual(None,self.log._get_next_line())

    def test_parse_header(self):
        """Parse the header of a MySQL log file"""
        # Tested with test___init__
        self.assertRaises(LogParserError,self.log._parse_header,"Ham Spam")
        self.assertEqual(None,self.log._parse_header(None))
    
    def test_version(self):
        """Get the version of the MySQL server"""
        exp = (5,5,17,'log')
        self.log._version = exp
        self.assertEqual(exp,self.log.version)
    
    def test_program(self):
        """Get the full path of the MySQL server executable"""
        exp = '/usr/sbin/mysqld'
        self.log._program = exp
        self.assertEqual(exp,self.log.program)

    def test_port(self):
        """Get the TCP/IP port of the MySQL server"""
        exp = 3309
        self.log._port = exp
        self.assertEqual(exp,self.log.port)
    
    def test_socket(self):
        """Get the Unix socket of the MySQL server"""
        exp = '/tmp/mysql.sock'
        self.log._socket = exp
        self.assertEqual(exp,self.log.socket)

    def test_start_datetime(self):
        """Get the timestamp of the first read log entry"""
        exp = datetime.datetime(2011,11,2,12,48,46)
        self.log._start_datetime = exp
        self.assertEqual(exp,self.log.start_datetime)

    def test_last_seen_datetime(self):
        """Get the timestamp of the last read log entry"""
        exp = datetime.datetime(2011,11,2,12,48,46)
        self.log._last_seen_datetime = exp
        self.assertEqual(exp,self.log.last_seen_datetime)
    
    def test_str(self):
        """String representation of LogParserBase"""
        version = (5,5,17,'-log')
        self.log._version = version
        exp = "<LogParserBase, MySQL v5.5.17-log>"
        self.assertEqual(exp,str(self.log))
        
        version = (5,5,17,None)
        self.log._version = version
        exp = "<LogParserBase, MySQL v5.5.17>"
        self.assertEqual(exp,str(self.log))
        

class TestGeneralQueryLog(BaseParserTestCase):
    def setUp(self):
        self.fakelog = tempfile.SpooledTemporaryFile(max_size=3*1024)
        self.log = GeneralQueryLog(stream=self.fakelog)
    
    def tearDown(self):
        try:
            self.fakelog.close()
        except:
            pass
    
    def test_init(self):
        """Initialzing GeneralQueryLog instance (empty log)"""
        self.assertRaises(TypeError,GeneralQueryLog)

        exp = {
            '_version': None,
            '_program': None,
            '_port': None,
            '_socket': None,
            '_start_datetime': None,
            '_last_seen_datetime': None,
            '_sessions': {},
            '_cached_logentry': None,
        }
        self.check_attributes(self.log,exp)

        self._fakelog_writelines(LOG_HEADERS[0], empty=True)
        exp = {
            '_stream': self.fakelog,
            '_version': (5,1,54,"1ubuntu4-log"),
            '_program': '/usr/sbin/mysqld',
            '_port': 3309,
            '_socket': '/var/run/mysqld/mysqld3.sock',
            '_start_datetime': None,
            '_last_seen_datetime': None,
        }
        self.log = GeneralQueryLog(self.fakelog)
        self.check_attributes(self.log,exp)
    
    def test__new_session(self):
        """Add a new session"""
        session_id = 1
        exp = dict(database=None,
                   user=None,
                   host=None,
                   time_last_action=None,
                   to_delete=False)
        self.log._new_session(session_id)
        self.assertEqual(exp,self.log._sessions[session_id])
    
    def test_parse_command(self):
        """Parse a line from General Query Log having a command"""
        self.assertEqual(None,self.log._parse_command(None,None))

    def test_handle_connect(self):
        """Parse General Query Log Connect entries: Connect-Command"""
        lines = []
        for line in GENERAL_LOG_ENTRIES['Connect']:
            lines += line.splitlines()
        self._fakelog_writelines(lines)

        i = 0
        logentry = self.fakelog. readline().strip()
        while logentry:
            exp = GENERAL_LOG_ENTRIES_EXP['Connect'][i]
            entry = GeneralQueryLogEntry()
            self.log._parse_command(logentry,entry)
            self.assertEqual(exp,dict(entry))
            result = self.log._sessions[entry['session_id']]['database']
            if entry['argument'].endswith('on'):
                self.assertEqual(None,result)
            else:
                self.assertEqual(entry['argument'].split(' ')[2],result)
            if self.log._cached_logentry is not None:
                logentry = self.log._cached_logentry
            else:
                logentry = self.fakelog. readline().strip()
            i += 1
    
    def _check_simple_commands(self, command):
        lines = []
        for line in GENERAL_LOG_ENTRIES[command]:
            lines += line.splitlines()
        self._fakelog_writelines(lines)

        i = 0
        logentry = self.fakelog.readline().strip()
        while logentry:
            exp = GENERAL_LOG_ENTRIES_EXP[command][i]
            entry = GeneralQueryLogEntry()
            self.log._parse_command(logentry,entry)
            self.assertEqual(exp,dict(entry))
            if self.log._cached_logentry is not None:
                logentry = self.log._cached_logentry
            else:
                logentry = self.fakelog.readline().strip()
            i += 1
    
    def test_handle_multi_line(self):
        """Parse General Query Log entries spanning multiple lines"""
        self._check_simple_commands("Query")
        self._check_simple_commands("Prepare")

    def test_handle_init_db(self):
        """Parse General Query Log Query entries: Init DB-command"""
        self._check_simple_commands("Init DB")
    
    def test_handle_quit(self):
        """Parse General Query Log Query entries: Quit-command"""
        self._check_simple_commands("Quit")

    def test_parse_entry(self):
        """"Parse a General Query Log entry"""
        self.assertEqual(None,self.log._parse_entry())

        # Parsing headers while getting entries
        self.log = GeneralQueryLog(stream=self.fakelog)
        for i,data in enumerate(LOG_HEADERS):
            self._fakelog_writelines(data, empty=True)
            self.log._parse_entry()
            self.check_attributes(self.log,LOG_HEADERS_EXP[i])

    def test_iter(self):
        """Iterate through the log, reading entries (iter()-method)"""
        try:
            iter(self.log)
        except TypeError:
            self.fail("GeneralQueryLog is not iterable")
        
        self._fakelog_writelines(GENERAL_LOG_ENTRIES['Query'],
                                 empty=True,rewind=True)

        log_iterator = self.log.__iter__()
        for i,exp in enumerate(GENERAL_LOG_ENTRIES_EXP['Query']):
            entry = log_iterator.next()
            self.assertTrue(isinstance(entry, GeneralQueryLogEntry))
            self.assertEqual(exp,entry,msg="Failed parsing entry #%d" % i)

    def test_next(self):
        """Iterate through the log, reading entries (.next()-method)"""
        self.assertRaises(StopIteration,self.log.next)

        data = GENERAL_LOG_ENTRIES['Query'][0]
        exp = GENERAL_LOG_ENTRIES_EXP['Query'][0]
        self._fakelog_writelines([data],empty=True,rewind=True)
        self.assertEqual(exp,self.log.next())
        self.assertRaises(StopIteration,self.log.next)

class TestSlowQueryLog(BaseParserTestCase):
    """Test SlowQueryLog class"""

    def setUp(self):
        self.fakelog = tempfile.SpooledTemporaryFile(max_size=3*1024)
        self.log = SlowQueryLog(stream=self.fakelog)
    
    def tearDown(self):
        try:
            self.fakelog.close()
        except:
            pass

    def test_init(self):
        """Initialzing SlowQueryLog instance (empty log)"""
        exp = {
            '_stream': self.fakelog,
            '_version': None,
            '_program': None,
            '_port': None,
            '_socket': None,
            '_start_datetime': None,
            '_last_seen_datetime': None,
            '_cached_line': None,
            '_current_database': None,
        }
        self.check_attributes(self.log,exp)

        self._fakelog_writelines(LOG_HEADERS[0], empty=True)
        exp = {
            '_stream': self.fakelog,
            '_version': (5,1,54,"1ubuntu4-log"),
            '_program': '/usr/sbin/mysqld',
            '_port': 3309,
            '_socket': '/var/run/mysqld/mysqld3.sock',
            '_start_datetime': None,
            '_last_seen_datetime': None,
        }
        self.log = SlowQueryLog(self.fakelog)
        self.check_attributes(self.log,exp)

    def test_parse_line(self):
        """Parse a log line using regular expression Match-object"""
        regex = re.compile("(\d{1}) (\d{2}) (\d{3}) (\w{3})")
        line = "1 12 123 ham"
        exp = ('1','12','123','ham')
        self.assertEqual(exp,self.log._parse_line(regex, line))

        line = "spam"
        self.assertRaises(LogParserError, self.log._parse_line, regex, line)
    
    def test_parse_connection_info(self):
        """Get connection info from a slow query log entry"""
        tests = [
            ('# User@Host: root[root] @ localhost []',
             dict(user='root', host='localhost', session_id=None)),
            ('# User@Host: root[root] @ localhost [127.0.0.1]',
             dict(user='root', host='localhost', session_id=None)),
            ('# User@Host: root[root] @ [127.0.0.1]',
             dict(user='root', host='127.0.0.1', session_id=None)),
            ('# User@Host: [ham] @ localhost []',
             dict(user='ham', host='localhost', session_id=None)),
            ('# User@Host: [ham] @ [127.0.0.1]',
             dict(user='ham', host='127.0.0.1', session_id=None)),
        ]
        for data,exp in tests:
            result = dict()
            try:
                self.log._parse_connection_info(data,result)
            except LogParserError:
                self.fail("Failed parsing (correct) line: '%s'" % data)
            self.assertEqual(exp,result)

        self.assertRaises(LogParserError,self.log._parse_connection_info,
            'ham spam ham spam',dict())
    
    def test_parse_timestamp(self):
        """Get the timestamp of the slow query log entry"""
        tests = [
            ('# Time: 111102 12:48:46',
             dict(datetime=datetime.datetime(2011,11,2,12,48,46))),
            ('# Time: 111102   9:48:46',
             dict(datetime=datetime.datetime(2011,11,2,9,48,46))),
        ]
        for data,exp in tests:
            result = dict()
            try:
                self.log._parse_timestamp(data,result)
            except LogParserError:
                self.fail("Failed parsing (correct) line: '%s'" % data)
            self.assertEqual(exp,result)

        self.assertRaises(LogParserError,self.log._parse_timestamp,
            'ham spam ham spam',dict())
    
    def test_parse_statistics(self):
        """Get statistics of the slow query log entry"""
        tests = [
            ('# Query_time: 0.000333  Lock_time: 0.000000'\
             ' Rows_sent: 1  Rows_examined: 0',
                dict(query_time=decimal.Decimal('0.000333'),
                     lock_time=decimal.Decimal('0.000000'),
                     rows_examined=0,rows_sent=1)),
            ('# Query_time: 123.000333  Lock_time: 323.000001'\
             ' Rows_sent: 12  Rows_examined: 34',
                dict(query_time=decimal.Decimal('123.000333'),
                     lock_time=decimal.Decimal('323.000001'),
                     rows_examined=34,rows_sent=12)),
        ]
        for data,exp in tests:
            result = dict()
            try:
                self.log._parse_statistics(data,result)
            except LogParserError:
                self.fail("Failed parsing (correct) line: '%s'" % data)
            self.assertEqual(exp,result)
        
        self.assertRaises(LogParserError,self.log._parse_statistics,
            'ham spam ham spam',dict())
    
    def test_parse_query(self):
        """Get the queries of a slow query log entry"""
        tests = [
            # First entry should have no 'use'-statement
            (   ('SET timestamp=1320234526;',
                 'select @@version_comment limit 1;'),
                dict(database=None,
                     datetime=datetime.datetime.fromtimestamp(1320234526),
                     query='SET timestamp=1320234526;\n'\
                        'select @@version_comment limit 1;')),
            
            (   ('SET timestamp=1320234526;',
                 'select c1',
                 'from t1',
                 'where c1 like "%a%";'),
                dict(database=None,
                     datetime=datetime.datetime.fromtimestamp(1320234526),
                     query='SET timestamp=1320234526;\n'\
                           'select c1\nfrom t1\nwhere c1 like "%a%";')),
            
            # Last entry should have 'use test'
            (   ('use test',
                 'SET timestamp=1320234526;',
                 'select @@version_comment limit 1;'),
                dict(database='test',
                     datetime=datetime.datetime.fromtimestamp(1320234526),
                     query='use test\nSET timestamp=1320234526;\n'\
                           'select @@version_comment limit 1;')),
        ]

        for data,exp in tests:
            self._fakelog_writelines(data, empty=True)
            result = dict(database=None, query=None)
            try:
                self.log._parse_query(self.log._get_next_line(),result)
            except LogParserError:
                self.fail("Failed parsing (correct) line: '%s'" % data)
            self.assertEqual(exp,result)
        
        # Database should be same as last item in the tests-list
        self._fakelog_writelines(tests[0][0], empty=True)
        result = dict(database=None, query=None)
        exp = dict(database='test',
                   datetime=datetime.datetime.fromtimestamp(1320234526),
                   query='SET timestamp=1320234526;\n'\
                         'select @@version_comment limit 1;')
        try:
            self.log._parse_query(self.log._get_next_line(),result)
        except LogParserError:
            self.fail("Failed parsing (correct) line: '%s'" % data)
        self.assertEqual(exp,result)
    
    def test_parse_entry(self):
        """Parse a Slow Query Log entry"""
        self.assertEqual(None,self.log._parse_entry())

        for i,data in enumerate(SLOW_LOG_ENTRIES):
            self._fakelog_writelines(data, empty=True)
            self.assertEqual(SLOW_LOG_ENTRIES_EXP[i],
                             self.log._parse_entry())
        
        # Parsing headers while getting entries
        self.log = SlowQueryLog(stream=self.fakelog)
        for i,data in enumerate(LOG_HEADERS):
            self._fakelog_writelines(data, empty=True)
            self.log._parse_entry()
            self.check_attributes(self.log,LOG_HEADERS_EXP[i])

    def test_iter(self):
        """Iterate through the log, reading entries (iter()-method)"""
        try:
            iter(self.log)
        except TypeError:
            self.fail("SlowQueryLog is not iterable")

        for i,data in enumerate(SLOW_LOG_ENTRIES):
            self._fakelog_writelines(data, empty=False,rewind=False)
        self.fakelog.seek(0)

        log_iterator = self.log.__iter__()
        for i,exp in enumerate(SLOW_LOG_ENTRIES_EXP):
            entry = log_iterator.next()
            self.assertTrue(isinstance(entry, SlowQueryLogEntry))
            self.assertEqual(exp,entry,msg="Failed parsing entry #%d" % i)
    
    def test_next(self):
        """Iterate through the log, reading entries (.next()-method)"""
        self.assertRaises(StopIteration,self.log.next)

        data = SLOW_LOG_ENTRIES[0]
        exp = SLOW_LOG_ENTRIES_EXP[0]
        self._fakelog_writelines(data,empty=True,rewind=True)
        self.assertEqual(exp,self.log.next())
        self.assertRaises(StopIteration,self.log.next)

class TestLogEntryBase(BaseParserTestCase):
    entry_init_attributes =  {
        'datetime': None,
        'database': None,
        'user': None,
        'host': None,
    }

    def setUp(self):
        self.entry = LogEntryBase()

    def test_init(self):
        """Initialize LogEntryBase"""
        self.check_attributes(self.entry,self.entry_init_attributes)

class TestGeneralQueryLogEntry(BaseParserTestCase):
    entry_init_attributes =  {
        'datetime': None,
        'database': None,
        'user': None,
        'host': None,
        'session_id': None,
        'command': None,
        'argument': None,
    }

    def setUp(self):
        self.entry = GeneralQueryLogEntry()

    def test_init(self):
        """Initialize LogEntryBase"""
        self.check_attributes(self.entry,self.entry_init_attributes)
    
    def test___str__(self):
        """String representation of LogParserBase"""
        exp = "<GeneralQueryLogEntry  [None] None: None>"
        self.assertEqual(exp,str(self.entry))

class TestSlowQueryLogEntry(BaseParserTestCase):
    entry_init_attributes =  {
        'datetime': None,
        'database': None,
        'query': None,
        'user': None,
        'host': None,
        'query_time': None,
        'lock_time': None,
        'rows_examined': None,
        'rows_sent': None,
    }

    def setUp(self):
        self.entry = SlowQueryLogEntry()

    def test_init(self):
        """Initialize LogEntryBase"""
        self.check_attributes(self.entry,self.entry_init_attributes)
    
    def test_str(self):
        """String representation of LogParserBase"""
        exp = "<SlowQueryLogEntry  [None@None] None/None/None/None>"
        self.assertEqual(exp,str(self.entry))

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
"""Setup script for MySQL Utilities"""
from __future__ import absolute_import

import os
import re
from glob import glob
import sys

import distutils.core
from distutils.core import setup
from distutils.command.build_scripts import build_scripts as _build_scripts
from distutils.command.install import install as _install
from distutils.command.install_scripts import \
    install_scripts as _install_scripts
from distutils.util import change_root
from distutils.file_util import DistutilsFileError, write_file
from distutils import log, dir_util

from info import META_INFO, INSTALL

# Check required Python version
if sys.version_info[0:2] not in [(2, 6), (2, 7)]:
    log.error("MySQL Utilities requires Python v2.6 or v2.7")
    sys.exit(1)

COMMANDS = {
    'cmdclass': {
        },
    }

# Custom bdist_rpm DistUtils command
try:
    from support.dist_rpm import BuiltDistRPM, BuiltCommercialRPM, SourceRPM
except ImportError:
    pass # Use default when not available
else:
    COMMANDS['cmdclass'].update({
        'bdist_rpm': BuiltDistRPM,
        'sdist_rpm': SourceRPM,
        'bdist_com_rpm': BuiltCommercialRPM
    })

try:
    from support.distribution.commands import build, bdist, sdist
except ImportError:
    pass # Use default when not available
else:
    COMMANDS['cmdclass'].update({
        'build': build.Build,
        'sdist_com': sdist.SourceCommercial,
        'bdist_com': bdist.BuiltCommercial
    })

try:
    from support.dist_deb import BuildDistDebian, BuildCommercialDistDebian
                                  
except ImportError:
    pass
else:
    COMMANDS['cmdclass'].update({
        'bdist_deb': BuildDistDebian,
        'bdist_com_deb': BuildCommercialDistDebian
    })
ARGS = {
}

PROFILE_SCRIPT = '''
prepend_path () (
    IFS=':'
    for D in $PATH; do
        if test x$D != x$1; then
            OUTPATH="${OUTPATH:+$OUTPATH:}$D"
        fi
    done
    echo "$1:$OUTPATH"
)

PATH=`prepend_path %s`
'''


class install(_install):
    """Install MySQL Utilities"""
    user_options = _install.user_options + [
        ("skip-profile", None, "Skip installing a profile script"),
        ]

    boolean_options = _install.boolean_options + ['skip-profile']

    def initialize_options(self):
        """Initialize options"""
        _install.initialize_options(self)
        self.skip_profile = False

    def finalize_options(self):
        """Finalize options"""
        _install.finalize_options(self)

    def run(self):
        _install.run(self)


class install_man(distutils.core.Command):
    description = "Install Unix manual pages"

    user_options = [
        ('prefix=', None, 'installation prefix (default /usr/share/man)'),
        ('root=', None,
         "install everything relative to this alternate root directory"),
        ('record=', None,
         "filename in which to record list of installed files"),
    ]

    def initialize_options(self):
        """Initialize options"""
        self.root = None
        self.prefix = None
        self.record = None

    def finalize_options(self):
        """Finalize options"""
        self.set_undefined_options('install',
                                   ('root', 'root'),
                                   ('record', 'record')
                                   )
        if not self.prefix:
            self.prefix = '/usr/share/man'

        if self.root:
            self.prefix = change_root(self.root, self.prefix)

    def run(self):
        """Run the command"""
        srcdir = os.path.join('docs', 'man')
        manpages = os.listdir(srcdir)
        self._outfiles = []
        for man in manpages:
            src_man = os.path.join(srcdir, man)
            section = os.path.splitext(man)[1][1:]
            dest_dir = os.path.join(self.prefix, 'man' + section)
            self.mkpath(dest_dir) # Could be different section
            dest_man = os.path.join(dest_dir, man)
            self.copy_file(src_man, dest_man)
            self._outfiles.append(dest_man)

        # Disabled, done in the RPM spec
        #self._write_record()

    def _write_record(self):
        """Write list of installed files"""
        if self.record:
            outputs = self.get_outputs()
            if self.root:               # strip any package prefix
                root_len = len(self.root)
                for counter in xrange(len(outputs)):
                    outputs[counter] = outputs[counter][root_len:]

            log.info("writing list of installed files to '{0}'".format(
                self.record))
            f = open(self.record, "a")
            for line in outputs:
                f.write(line + "\n")

    def get_outputs(self):
        return self._outfiles


class install_scripts(_install):
    """Install MySQL Utilities scripts"""
    description = "Install the Shell Profile (Linux/Unix)"

    user_options = _install.user_options + [
        ('root=', None,
         "install everything relative to this alternate root directory"),
        ]

    boolean_options = _install.boolean_options + ['skip-profile']
    profile_filename = 'mysql-utilities.sh'
    profile_d_dir = '/etc/profile.d/'

    def initialize_options(self):
        """initialize options"""
        _install.initialize_options(self)
        self.skip_profile = False
        self.root = None
        self.install_dir = None

    def finalize_options(self):
        """Finalize options"""
        _install.finalize_options(self)
        self.set_undefined_options('install',
                                   ('install_dir', 'install_dir'),
                                   ('root', 'root'))

    def _create_shell_profile(self):
        """Creates and installes the shell profile

        This method will create and try to install the shell
        profile file under /etc/profile.d/. It will skip this
        step when the --skip-profile install option has been
        given, or when the user installing MySQL Utilities
        has no permission.
        """
        if self.skip_profile:
            log.info("Not adding shell profile %s (skipped)" % (
                     os.path.join(self.profile_d_dir, self.profile_filename)))
            return

        if self.root:
            profile_dir = change_root(self.root, self.profile_d_dir)
        else:
            profile_dir = self.profile_d_dir

        try:
            dir_util.mkpath(profile_dir)
        except DistutilsFileError as err:
            log.info("Not installing mysql-utilities.sh: {0}".format(err))
            self.skip_profile = True
            return

        destfile = os.path.join(profile_dir, self.profile_filename)
        if not os.access(os.path.dirname(destfile), os.X_OK | os.W_OK):
            log.info("Not installing mysql-utilities.sh in "
                     "{folder} (no permission)".format(folder=destfile))
            self.skip_profile = True
            return

        if os.path.exists(os.path.dirname(destfile)):
            if os.path.isdir(destfile) and not os.path.islink(destfile):
                dir_util.remove_tree(destfile)
            elif os.path.exists(destfile):
                log.info("Removing {filename}".format(filename=destfile))
                os.unlink(destfile)

        script = PROFILE_SCRIPT % (self.install_dir,)
        log.info("Writing {filename}".format(filename=destfile))
        open(destfile, "w+").write(script)

    def run(self):
        """Run the command"""
        self._create_shell_profile()

    def get_outputs(self):
        """Get installed files"""
        outputs = _install.get_outputs(self)
        return outputs


class build_scripts(_build_scripts):
    """Class for providing a customized version of build_scripts.

    When ``run`` is called, this command class will:
    1. Create a copy of all ``.py`` files in the **scripts** option
       that does not have the ``.py`` extension.
    2. Replace the list in the **scripts** attribute with a list
       consisting of the script files with the ``.py`` extension
       removed.
    3. Call run method in `distutils.command.build_scripts`.
    4. Restore the scripts list to the old value, for other commands
       to use."""

    def run(self):
        if not self.scripts:
            return

        saved_scripts = self.scripts
        self.scripts = []
        for script in saved_scripts:
            script = distutils.util.convert_path(script)
            script_copy, script_ext = os.path.splitext(script)

            if script_ext != '.py':
                log.debug("Not removing extension from {script} "
                          "since it's not '.py'".format(script=script))
            else:
                log.debug("Copying {orig} -> {dest}".format(
                    orig=script, dest=script_copy))
                self.copy_file(script, script_copy)
                self.scripts.append(script_copy)
        # distutils is compatible with 2.1 so we cannot use super() to
        # call it.
        _build_scripts.run(self)
        self.outfiles = self.scripts
        self.scripts = saved_scripts
        

    def get_outputs(self):
        """Get installed files"""
        return self.outfiles

COMMANDS['cmdclass'].update({
        'install': install,
        })

if os.name != "nt":
    COMMANDS['cmdclass'].update({
        'build_scripts': build_scripts,
        'install_man': install_man,
        })

ARGS.update(META_INFO)
ARGS.update(INSTALL)
ARGS.update(COMMANDS)
setup(**ARGS)




#
# Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

import distutils.command.build_scripts
import distutils.util
import glob
import os

import mysql.utilities

META_INFO = {
    'name': 'mysql-utilities',
    'description': 'MySQL Utilities ' + mysql.utilities.RELEASE_STRING,
    'maintainer': 'Oracle',
    'maintainer_email': '',
    'version': mysql.utilities.VERSION_STRING,
    'url': 'http://dev.mysql.com',
    'license': 'GNU GPLv2 (with FOSS License Exception)',
    'keywords': "mysql db",
    'classifiers': [
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Environment :: Console',
        'Environment :: Win32 (MS Windows)',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Database Administrators',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: OS Independent',
        'Operating System :: POSIX',
        'Topic :: Utilities',
        ],
    }

INSTALL = {
    'packages': [
        'mysql',
        'mysql.utilities',
        'mysql.utilities.command',
        'mysql.utilities.common',
        ],
    'scripts': glob.glob('scripts/*.py'),
    'requires': [
        'distutils',
        'sphinx (>=1.0)',
        'jinja2 (>=2.1)',
        ],
    'provides': [
        'mysql.utilities',
        ],
    }



#!/usr/bin/env python
#
# Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

"""
This file contains the clone server utility which launches a new instance
of an existing server.
"""

from mysql.utilities.common.tools import check_python_version

# Check Python version compatibility
check_python_version()

import os.path
import sys

from mysql.utilities.common.options import add_basedir_option
from mysql.utilities.common.options import check_basedir_option
from mysql.utilities.common.ip_parser import parse_connection
from mysql.utilities.common.options import setup_common_options
from mysql.utilities.common.options import add_verbosity
from mysql.utilities.common.tools import check_connector_python
from mysql.utilities import exception
from mysql.utilities.command import serverclone

# Constants
NAME = "MySQL Utilities - mysqlserverclone "
DESCRIPTION = "mysqlserverclone - start another instance of a running server"
USAGE = "%prog --server=user:pass@host:port:socket --new-data=/tmp/data2 " \
        "--new-port=3310 --new-id=12 --root-password=root"

# Check for connector/python
if not check_connector_python():
    sys.exit(1)

# Setup the command parser and setup server, help
parser = setup_common_options(os.path.basename(sys.argv[0]),
                              DESCRIPTION, USAGE, False, True, None)

# Setup utility-specific options:

# Data directory for new instance
parser.add_option("--new-data", action="store", dest="new_data",
                  type = "string", help="the full path to the location "
                  "of the data directory for the new instance")

# Port for the new instance
parser.add_option("--new-port", action="store", dest="new_port",
                  type = "string", default="3307", help="the new port "
                         "for the new instance - default=%default")

# Server id for the new instance
parser.add_option("--new-id", action="store", dest="new_id",
                  type = "string", default="2", help="the server_id for "
                         "the new instance - default=%default")

# Root password for the new instance
parser.add_option("--root-password", action="store", dest="root_pass",
                  type="string", help="password for the root user")

# Optional additional command-line options
parser.add_option("--mysqld", action="store", dest="mysqld",
                  type="string", help="additional options for mysqld")

# Option to write command to file
parser.add_option("--write-command", "-w", action="store", dest='cmd_file',
                  default=None, type="string", help="path to file for writing"
                  " startup command. For example: start_server1.sh")

# Add verbosity and quiet mode
add_verbosity(parser, True)

# Add --basedir option
add_basedir_option(parser)

# Add --delete-data
parser.add_option("--delete-data", action="store_true", dest="delete",
                  help="delete the folder specified by --new-data if it "
                  "exists and is not empty.")

# Add user option
parser.add_option("--user", action="store", dest="user", type="string",
                  default=None, help="user account to launch cloned server. "
                  "Default is current user.")

# Add startup timeout
parser.add_option("--start-timeout", action="store", dest="start_timeout",
                  type=int, default=10, help="Number of seconds to wait for "
                  "server to start. Default = 10.")

# Now we process the rest of the arguments.
opt, args = parser.parse_args()

# Check the basedir option for errors (e.g., invalid path)
check_basedir_option(parser, opt.basedir)

# Can only use --basedir and --datadir if --server is missing
if opt.basedir is not None and opt.server is not None:
    parser.error("Cannot use the --basedir and --server options together.")

# Fail if no database path specified.
if opt.new_data is None:
    parser.error("No new database path. Use --help for available options.")

# Warn if root-password is left off.
if opt.root_pass is None or opt.root_pass == '':
    print("# WARNING: Root password for new instance has not been set.")

# Fail if user does not have access to new data dir.
if os.path.exists(opt.new_data):
    if not os.access(opt.new_data, os.R_OK|os.W_OK):
        parser.error("You do not have enough privileges to access the folder "
                     "specified by --new-data.")

    # Fail if new data is not empty and delete not specified
    if os.listdir(opt.new_data) and not opt.delete:
        parser.error("Target data directory exists and is not empty. Use "
                     "--delete-data option to delete folder before cloning.")

# Check start timeout for minimal value
if int(opt.start_timeout) < 10:
    opt.start_timeout = 10
    print("# WARNING: --start-timeout must be >= 10 seconds. Using "
          "default value.")

# Build options
options = {
    'new_data'       : opt.new_data,
    'new_port'       : opt.new_port,
    'new_id'         : opt.new_id,
    'root_pass'      : opt.root_pass,
    'mysqld_options' : opt.mysqld,
    'verbosity'      : opt.verbosity,
    'quiet'          : opt.quiet,
    'cmd_file'       : opt.cmd_file,
    'basedir'        : opt.basedir,
    'delete'         : opt.delete,
    'user'           : opt.user,
    'start_timeout'  : opt.start_timeout,
}

# Expand user paths and resolve relative paths
if opt.new_data and opt.new_data[0] == '~':
    options['new_data'] = os.path.expanduser(opt.new_data)
if opt.basedir and opt.basedir[0] == '~':
    options['basedir'] = os.path.expanduser(opt.basedir)
if opt.new_data and opt.new_data[0] == '.':
    options['new_data'] = os.path.abspath(opt.new_data)
if opt.basedir and opt.basedir[0] == '.':
    options['basedir'] = os.path.abspath(opt.basedir)

# Parse source connection values if we have a running server
if opt.basedir is None:
    try:
        conn = parse_connection(opt.server)
    except exception.FormatError:
        _, err, _ = sys.exc_info()
        parser.error("Server connection values invalid: %s." % err)
    except exception.UtilError:
        _, err, _ = sys.exc_info()
        parser.error("Server connection values invalid: %s." % err.errmsg)
else:
    conn = None

try:
    res = serverclone.clone_server(conn, options)
except exception.UtilError:
    _, e, _ = sys.exc_info()
    print("ERROR: %s" % e.errmsg)
    sys.exit(1)

sys.exit()



#!/usr/bin/env python
#
# Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

"""
This file contains the replicate utility. It is used to establish a
master/slave replication topology among two servers.
"""

from mysql.utilities.common.tools import check_python_version

# Check Python version compatibility
check_python_version()

import os.path
import sys

from mysql.utilities.command.setup_rpl import setup_replication
from mysql.utilities.common.ip_parser import parse_connection
from mysql.utilities.common.messages import (PARSE_ERR_OPTS_REQ,
                                             WARN_OPT_USING_DEFAULT)
from mysql.utilities.common.options import setup_common_options
from mysql.utilities.common.options import add_verbosity, add_rpl_user
from mysql.utilities.common.server import check_hostname_alias
from mysql.utilities.common.tools import check_connector_python
from mysql.utilities.exception import FormatError
from mysql.utilities.exception import UtilError

# Constants
NAME = "MySQL Utilities - mysqlreplicate "
DESCRIPTION = "mysqlreplicate - establish replication with a master"
USAGE = "%prog --master=root@localhost:3306 --slave=root@localhost:3310 " \
        "--rpl-user=rpl:passwd "

# Check for connector/python
if not check_connector_python():
    sys.exit(1)

# Setup the command parser
parser = setup_common_options(os.path.basename(sys.argv[0]),
                              DESCRIPTION, USAGE, True, False)

# Setup utility-specific options:

# Connection information for the source server
parser.add_option('--master', action="store", dest="master",
                  type="string", default=None,
                  help="connection information for master server in "
                  "the form: <user>[:<password>]@<host>[:<port>][:<socket>] or"
                  " <login-path>[:<port>][:<socket>].")

# Connection information for the destination server
parser.add_option('--slave', action="store", dest="slave",
                  type="string", default=None,
                  help="connection information for slave server in "
                  "the form: <user>[:<password>]@<host>[:<port>][:<socket>] or"
                  " <login-path>[:<port>][:<socket>].")

# Replication user and password
add_rpl_user(parser)

# Pedantic mode for failing if storage engines differ
parser.add_option("-p", "--pedantic", action="store_true", default=False,
                  dest="pedantic", help="fail if storage engines differ "
                  "among master and slave.")

# Test replication option
parser.add_option("--test-db", action="store", dest="test_db",
                  type="string", help="database name to use in testing "
                  "replication setup (optional)")

# Add master log file option
parser.add_option("--master-log-file", action="store", dest="master_log_file",
                  type="string", help="use this master log file to initiate "
                  "the slave.", default=None)

# Add master log position option
parser.add_option("--master-log-pos", action="store", dest="master_log_pos",
                  type="int", help="use this position in the master log file "
                  "to initiate the slave.", default=-1)

# Add start from beginning option
parser.add_option("-b", "--start-from-beginning", action="store_true",
                  default=False, dest="from_beginning",
                  help="start replication from the first event recorded in "
                  "the binary logging of the master."
                  "Not valid with --master-log-file or --master-log-pos.")

# Add verbosity
add_verbosity(parser)

# Now we process the rest of the arguments.
opt, args = parser.parse_args()

# option --master is required (mandatory)
if not opt.master:
    default_val = 'root@localhost:3306'
    print(WARN_OPT_USING_DEFAULT.format(default=default_val, opt='--master'))
    # Print the WARNING to force determinism if a parser error occurs.
    sys.stdout.flush()

# option --slave is required (mandatory)
if not opt.slave:
    parser.error(PARSE_ERR_OPTS_REQ.format(opt='--slave'))

# Parse source connection values
try:
    m_values = parse_connection(opt.master, None, opt)
except FormatError:
    _, err, _ = sys.exc_info()
    parser.error("Master connection values invalid: %s." % err)
except UtilError:
    _, err, _ = sys.exc_info()
    parser.error("Master connection values invalid: %s." % err.errmsg)


# Parse source connection values
try:
    s_values = parse_connection(opt.slave, None, opt)
except FormatError:
    _, err, _ = sys.exc_info()
    parser.error("Slave connection values invalid: %s." % err)
except UtilError:
    _, err, _ = sys.exc_info()
    parser.error("Slave connection values invalid: %s." % err.errmsg)

# Check hostname alias
if check_hostname_alias(m_values, s_values):
    parser.error("The master and slave are the same host and port.")

# Check required --master-log-file for --master-log-pos
if (opt.master_log_pos >= 0 and opt.master_log_file is None):
    parser.error("You must specify a master log file to use the master "
                 "log file position option.")

if ((opt.master_log_pos >= 0) or (opt.master_log_file is not None)) and \
   opt.from_beginning:
    parser.error("The --start-from-beginning option is not valid in "
                 "combination with --master-log-file or --master-log-pos.")

# Create dictionary of options
options = {
    'verbosity'       : opt.verbosity,
    'pedantic'        : opt.pedantic,
    'quiet'           : False,
    'master_log_file' : opt.master_log_file,
    'master_log_pos'  : opt.master_log_pos,
    'from_beginning'  : opt.from_beginning,
}

try:
    res = setup_replication(m_values, s_values, opt.rpl_user,
                            options, opt.test_db)
except UtilError:
    _, e, _ = sys.exc_info()
    print("ERROR: %s" % e.errmsg)
    sys.exit(1)

sys.exit()



#!/usr/bin/python
#
# Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

from mysql.utilities.common.tools import check_python_version

# Check Python version compatibility
check_python_version()

import os.path
import sys

from mysql.utilities.command.proc import KILL_CONNECTION
from mysql.utilities.command.proc import KILL_QUERY
from mysql.utilities.command.proc import PRINT_PROCESS
from mysql.utilities.command.proc import USER, HOST, DB, COMMAND, INFO, STATE
from mysql.utilities.command.proc import ProcessGrep

from mysql.utilities.exception import EmptyResultError
from mysql.utilities.common.options import add_regexp
from mysql.utilities.common.options import setup_common_options
from mysql.utilities.common.options import add_verbosity
from mysql.utilities.common.options import add_format_option
from mysql.utilities.common.tools import check_connector_python


def add_pattern(option, opt, value, parser, field):
    entry = (field, value)
    try:
        getattr(parser.values, option.dest).append(entry)
    except AttributeError:
        setattr(parser.values, option.dest, [entry])

# Check for connector/python
if not check_connector_python():
    sys.exit(1)

# Setup the command parser and setup server, help
parser = setup_common_options(os.path.basename(sys.argv[0]),
                              "mysqlprocgrep - search process information",
                              "%prog --server=user:pass@host:port:socket "
                              "[options]", True)
# Add regexp
add_regexp(parser)

parser.add_option(
    "-Q", "--print-sql", "--sql",
    dest="print_sql", action="store_true", default=False,
    help="print the statement instead of sending it to the server. If a kill option is submitted, a procedure will be generated containing the code for executing the kill.")
parser.add_option(
    "--sql-body",
    dest="sql_body", action="store_true", default=False,
    help="only print the body of the procedure.")
parser.add_option(
    "--kill-connection",
    action="append_const", const=KILL_CONNECTION,
    dest="actions", default=[],
    help="kill all matching connections.")
parser.add_option(
    "--kill-query",
    action="append_const", const=KILL_QUERY,
    dest="actions", default=[],
    help="kill query for all matching processes.")
parser.add_option(
    "--print",
    action="append_const", const=PRINT_PROCESS,
    dest="actions", default=[],
    help="print all matching processes.")

# Output format
add_format_option(parser, "display the output in either grid (default), "
                  "tab, csv, or vertical format", "grid")

# Add verbosity mode
add_verbosity(parser, False)

# Adding the --match-* options
for col in USER, HOST, DB, COMMAND, INFO, STATE:
    parser.add_option(
        "--match-" + col.lower(),
        action="callback", callback=add_pattern, callback_args=(col,),
        dest="matches", type="string", metavar="PATTERN", default=[],
        help="match the '{0}' column of the PROCESSLIST table".format(col))

parser.add_option(
    "--age",
    dest="age", default=None,
    help="show only processes that have been in the current state more than "
         "a given time")

(options, args) = parser.parse_args()

# Print SQL if only --sql-body is given
if options.sql_body:
    options.print_sql = True

if (options.server is None or len(options.server) == 0) and not options.print_sql:
    parser.error("You need at least one server if you're not using the --sql option")
elif options.server is not None and len(options.server) > 0 and options.print_sql:
    parser.error("You should not include servers in the call if you are using the --sql option")

# If no option was supplied, we print the processes by default
if len(options.actions) == 0:
    options.actions.append(PRINT_PROCESS)

try:
    command = ProcessGrep(options.matches, options.actions, options.use_regexp,
                          age=options.age)
    if options.print_sql:
        print(command.sql(options.sql_body).strip())
    else:
        command.execute(options.server, format=options.format)
except EmptyResultError:
    _, details, _ = sys.exc_info()
    sys.stderr.write("No matches")
    sys.exit(1)
except Exception:
    _, details, _ = sys.exc_info()
    sys.stderr.write('ERROR: %s' % details)
    sys.exit(2)

sys.exit()



#!/usr/bin/env python
#
# Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

"""
This file contains the operations to perform database consistency checking
on two databases.
"""

from mysql.utilities.common.tools import check_python_version

# Check Python version compatibility
check_python_version()

import os
import re
import sys

from mysql.utilities.command.dbcompare import database_compare
from mysql.utilities.common.dbcompare import DEFAULT_SPAN_KEY_SIZE
from mysql.utilities.common.messages import (PARSE_ERR_DB_PAIR,
                                             PARSE_ERR_DB_PAIR_EXT,
                                             PARSE_ERR_DB_MISSING_CMP,
                                             PARSE_ERR_SPAN_KEY_SIZE_TOO_LOW)
from mysql.utilities.common.ip_parser import parse_connection
from mysql.utilities.common.options import add_difftype
from mysql.utilities.common.options import add_verbosity, check_verbosity
from mysql.utilities.common.options import add_changes_for, add_reverse
from mysql.utilities.common.options import add_format_option
from mysql.utilities.common.options import setup_common_options
from mysql.utilities.common.pattern_matching import REGEXP_OBJ_NAME
from mysql.utilities.common.sql_transform import is_quoted_with_backticks
from mysql.utilities.common.sql_transform import remove_backtick_quoting
from mysql.utilities.common.tools import check_connector_python

from mysql.utilities.exception import UtilError, FormatError

# Constants
NAME = "MySQL Utilities - mysqldbcompare "
DESCRIPTION = "mysqldbcompare - compare databases for consistency"
USAGE = "%prog --server1=user:pass@host:port:socket " + \
        "--server2=user:pass@host:port:socket db1:db2"
PRINT_WIDTH = 75

# Check for connector/python
if not check_connector_python():
    sys.exit(1)

# Setup the command parser
parser = setup_common_options(os.path.basename(sys.argv[0]),
                              DESCRIPTION, USAGE, server=False)

# Connection information for the source server
parser.add_option("--server1", action="store", dest="server1",
                  type="string", default="root@localhost:3306",
                  help="connection information for first server in "
                  "the form: <user>[:<password>]@<host>[:<port>][:<socket>]"
                  " or <login-path>[:<port>][:<socket>].")

# Connection information for the destination server
parser.add_option("--server2", action="store", dest="server2",
                  type="string", default=None,
                  help="connection information for second server in "
                  "the form: <user>[:<password>]@<host>[:<port>][:<socket>]"
                  " or <login-path>[:<port>][:<socket>].")

# Output format
add_format_option(parser, "display the output in either grid (default), "
                  "tab, csv, or vertical format", "grid")

# Add skips
parser.add_option("--skip-object-compare", action="store_true",
                  dest="no_object_check",
                  help="skip object comparison step")

parser.add_option("--skip-row-count", action="store_true",
                  dest="no_row_count",
                  help="skip row count step")

parser.add_option("--skip-diff", action="store_true",
                  dest="no_diff",
                  help="skip the object diff step")

parser.add_option("--skip-data-check", action="store_true",
                  dest="no_data",
                  help="skip data consistency check")

# Skip check of table options.
parser.add_option("--skip-table-options", action="store_true",
                  dest="skip_tbl_opts",
                  help="skip check of all table options (e.g., "
                       "AUTO_INCREMENT, ENGINE, CHARSET, etc.).")

# Add display width option
parser.add_option("--width", action="store", dest="width",
                  type="int", help="display width",
                  default=PRINT_WIDTH)

# run-all-tests mode
parser.add_option("-a", "--run-all-tests", action="store_true",
                  dest="run_all_tests",
                  help="do not abort when a diff test fails")

# turn off binlog mode
parser.add_option("--disable-binary-logging", action="store_true",
                  default="False", dest="toggle_binlog",
                  help="turn binary logging off during operation if enabled "
                  "(SQL_LOG_BIN=1). Note: may require SUPER privilege. "
                  "Prevents compare operations from being written to the "
                  "binary log.")

# turn off binlog mode
parser.add_option(
    "--span-key-size", action="store", default=DEFAULT_SPAN_KEY_SIZE,
    type="int", dest="span_key_size", help="changes the size of the key used"
    " for compare table contents. A higher value can help to get more "
    "accurate results comparing large databases, but may slow the algorithm."
    " Default value is {0}.".format(DEFAULT_SPAN_KEY_SIZE)
)

# Add verbosity and quiet (silent) mode
add_verbosity(parser, True)

# Add difftype option
add_difftype(parser, True)

# Add the direction (changes-for)
add_changes_for(parser)

# Add show reverse option
add_reverse(parser)

# Now we process the rest of the arguments.
opt, args = parser.parse_args()

# Warn if quiet and verbosity are both specified
check_verbosity(opt)

# Set options for database operations.
options = {
    "quiet"            : opt.quiet,
    "verbosity"        : opt.verbosity,
    "difftype"         : opt.difftype,
    "run_all_tests"    : opt.run_all_tests,
    "width"            : opt.width,
    "no_object_check"  : opt.no_object_check,
    "no_diff"          : opt.no_diff,
    "no_row_count"     : opt.no_row_count,
    "no_data"          : opt.no_data,
    "format"           : opt.format,
    "toggle_binlog"    : opt.toggle_binlog,
    "changes-for"      : opt.changes_for,
    "reverse"          : opt.reverse,
    "span_key_size"    : opt.span_key_size,
    "skip_table_opts"  : opt.skip_tbl_opts,
}

# Parse server connection values
server2_values = None
try:
    server1_values = parse_connection(opt.server1, None, options)
except FormatError:
    _, err, _ = sys.exc_info()
    parser.error("Server1 connection values invalid: %s." % err)
except UtilError:
    _, err, _ = sys.exc_info()
    parser.error("Server1 connection values invalid: %s." % err.errmsg)

if opt.server2:
    try:
        server2_values = parse_connection(opt.server2, None, options)
    except FormatError:
        _, err, _ = sys.exc_info()
        parser.error("Server2 connection values invalid: %s." % err)
    except UtilError:
        _, err, _ = sys.exc_info()
        parser.error("Server2 connection values invalid: %s." % err.errmsg)

# Check for arguments
if len(args) == 0:
    parser.error(PARSE_ERR_DB_MISSING_CMP)


if opt.span_key_size and opt.span_key_size < DEFAULT_SPAN_KEY_SIZE:
    parser.error(
        PARSE_ERR_SPAN_KEY_SIZE_TOO_LOW.format(
            s_value=opt.span_key_size, default=DEFAULT_SPAN_KEY_SIZE))
# Operations to perform:
# 1) databases exist
# 2) check object counts
# 3) check object differences
# 4) check row counts among the tables
# 5) check table data consistency

res = True
check_failed = False
arg_regexp = re.compile('{0}(?:(?:\:){0})?'.format(REGEXP_OBJ_NAME))
for db in args:
    # Split the database names considering backtick quotes
    grp = arg_regexp.match(db)
    if not grp:
        parser.error(PARSE_ERR_DB_PAIR.format(db_pair=db,
                                              db1_label='db1',
                                              db2_label='db2'))
    parts = grp.groups()
    matched_size = len(parts[0])
    if not parts[1]:
        parts = (parts[0], parts[0])
    else:
        # add 1 for the separator ':'
        matched_size = matched_size + 1
        matched_size = matched_size + len(parts[1])
    # Verify if the size of the databases matched by the REGEX is equal to the
    # initial specified string. In general, this identifies the missing use
    # of backticks.
    if matched_size != len(db):
        parser.error(PARSE_ERR_DB_PAIR_EXT.format(db_pair=db,
                                                  db1_label='db1',
                                                  db2_label='db2',
                                                  db1_value=parts[0],
                                                  db2_value=parts[1]))

    # Remove backtick quotes (handled later)
    db1 = remove_backtick_quoting(parts[0]) \
                if is_quoted_with_backticks(parts[0]) else parts[0]
    db2 = remove_backtick_quoting(parts[1]) \
                if is_quoted_with_backticks(parts[1]) else parts[1]

    try:
        res = database_compare(server1_values, server2_values,
                               db1, db2, options)
        print
    except UtilError:
        _, e, _ = sys.exc_info()
        print("ERROR: %s" % e.errmsg)
        sys.exit(1)

    if not res:
        check_failed = True

    if check_failed and not opt.run_all_tests:
        break

if not opt.quiet:
    print
    if check_failed:
        print("# Database consistency check failed.")
    else:
        sys.stdout.write("# Databases are consistent")
        if (opt.no_object_check or opt.no_diff or
            opt.no_row_count or opt.no_data or opt.skip_tbl_opts):
            sys.stdout.write(" given skip options specified")
        print(".")
    print("#\n# ...done")

if check_failed:
    sys.exit(1)

sys.exit()
